/*******************
 *
 *  piccel.c
 *
 *  support routines (outside glade-2) to load image data and manipulate profiles
 *
 ******************/

#include <glib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <lcms.h>
#include <gnome.h>

#include "piccel_defs.h"

#define d(x) ((double) (x))

GdkPixbuf *BaseImage, *ProfiledImage;
GError *gp_errorp = 0;

cmsHPROFILE inputProfile = 0;
cmsHPROFILE outputProfile = 0;
cmsHPROFILE monitorProfile = 0;
cmsHTRANSFORM transform = 0;


GdkPixbuf *
pLoadBaseImage (char *filename)
{
	BaseImage = gdk_pixbuf_new_from_file (filename, &gp_errorp);
	if (BaseImage == (GdkPixbuf *) NULL)
	{
		printf ("error loading base image: %s\n", gp_errorp->message);
		g_clear_error (&gp_errorp);
	}
	return BaseImage;
}

GdkPixbuf *
pLoadProfiledImage (char *filename)
{
	ProfiledImage = gdk_pixbuf_new_from_file (filename, &gp_errorp);
	if (ProfiledImage == (GdkPixbuf *) NULL)
	{
		printf ("error loading base image: %s\n", gp_errorp->message);
		g_clear_error (&gp_errorp);
	}
	return ProfiledImage;
}

void
pCloseInputProfile ()
{
	if (inputProfile)
		cmsCloseProfile (inputProfile);
	inputProfile = 0;
}

void
pCloseOutputProfile ()
{
	if (outputProfile)
		cmsCloseProfile (outputProfile);
	outputProfile = 0;
}

void
pCloseMonitorProfile ()
{
	if (monitorProfile)
		cmsCloseProfile (monitorProfile);
	monitorProfile = 0;
}

void
pCloseProfiles ()
{
	pCloseInputProfile ();
	pCloseOutputProfile ();
	pCloseMonitorProfile ();

	if (transform)
		cmsDeleteTransform (transform);
	transform = 0;
}


char *
pLoadInputProfile (char *filename)
{
	pCloseInputProfile ();
	inputProfile = cmsOpenProfileFromFile (filename, "r");
	if (inputProfile != (cmsHPROFILE) NULL)
	{
		return ((char *) cmsTakeProductName (inputProfile));
	}
	else
	{
		return ((char *) NULL);
	}
}

char *
pLoadOutputProfile (char *filename)
{
	pCloseOutputProfile ();
	outputProfile = cmsOpenProfileFromFile (filename, "r");
	if (outputProfile != (cmsHPROFILE) NULL)
	{
		return ((char *) cmsTakeProductName (outputProfile));
	}
	else
	{
		return ((char *) NULL);
	}
}

char *
pLoadMonitorProfile (char *filename)
{
	pCloseMonitorProfile ();
	monitorProfile = cmsOpenProfileFromFile (filename, "r");
	if (monitorProfile != (cmsHPROFILE) NULL)
	{
		return ((char *) cmsTakeProductName (monitorProfile));
	}
	else
	{
		return ((char *) NULL);
	}
}

char *
pProfilesRGB (char code)
{
	switch (code)
	{
	case 'i':
		inputProfile = cmsCreate_sRGBProfile ();
		return (char *) cmsTakeProductName (inputProfile);
		break;
	case 'o':
		outputProfile = cmsCreate_sRGBProfile ();
		return (char *) cmsTakeProductName (outputProfile);
		break;
	case 'm':
		monitorProfile = cmsCreate_sRGBProfile ();
		return (char *) cmsTakeProductName (monitorProfile);
		break;
	default:
		fprintf (stderr, "pProfilesRGB: bad input code %c\n", code);
		break;
	}

	return (char *) NULL;
}

void
pApplyTransform (GdkPixbuf * inpb, GdkPixbuf * outpb)
{
	guchar *in, *out;
	int rowstride, rowlen, rows, i;

	in = gdk_pixbuf_get_pixels (inpb);
	out = gdk_pixbuf_get_pixels (outpb);
	rowstride = gdk_pixbuf_get_rowstride (inpb);
	rowlen = gdk_pixbuf_get_width (inpb) *
		gdk_pixbuf_get_n_channels (inpb);
	rows = gdk_pixbuf_get_height (inpb);
	rows--;
	for (i = 0; i < rows; i++)
	{
		cmsDoTransform (transform, &(in[i * rowstride]),
				&(out[i * rowstride]), rowlen);
	}

}



static int
GetProfileColorSpace (cmsHPROFILE hProfile)
{
	icColorSpaceSignature ProfileSpace = cmsGetColorSpace (hProfile);

	switch (ProfileSpace)
	{

	case icSigGrayData:
		return PT_GRAY;
	case icSigRgbData:
		return PT_RGB;
	case icSigCmyData:
		return PT_CMY;
	case icSigCmykData:
		return PT_CMYK;
	case icSigYCbCrData:
		return PT_YCbCr;
	case icSigLuvData:
		return PT_YUV;
	case icSigXYZData:
		return PT_XYZ;
	case icSigLabData:
		return PT_Lab;
	case icSigLuvKData:
		return PT_YUVK;
	case icSigHsvData:
		return PT_HSV;
	case icSigHlsData:
		return PT_HLS;
	case icSigYxyData:
		return PT_Yxy;

	case icSigHexachromeData:
		return PT_HiFi;


	default:
		return icMaxEnumData;
	}
}

static DWORD
ComputeOutputFormatDescriptor (DWORD dwInput, int OutColorSpace)
{
	int IsPlanar = T_PLANAR (dwInput);
	int Channels = 0;


	switch (OutColorSpace)
	{

	case PT_GRAY:
		Channels = 1;
		break;
	case PT_RGB:
	case PT_CMY:
	case PT_Lab:
	case PT_YUV:
	case PT_YCbCr:
		Channels = 3;
		break;

	case PT_CMYK:
		Channels = 4;
		break;
	default:
		fprintf (stderr, "Unsupported output color space");
		exit (-1);
	}

	return (COLORSPACE_SH (OutColorSpace) | PLANAR_SH (IsPlanar) |
		CHANNELS_SH (Channels) | BYTES_SH (1));
}


void
pBuildTransform (int ip, int op, int mp, int gamut, int intent,
		 int proofIntent)
{

	DWORD wInput, wOutput;
	int OutputColorSpace;
	cmsHPROFILE tmp_inputProfile, tmp_outputProfile;

	if (transform)
		cmsDeleteTransform (transform);
	transform = 0;

	if (inputProfile == 0)
		inputProfile = cmsCreate_sRGBProfile ();
	if (outputProfile == 0)
		outputProfile = cmsCreate_sRGBProfile ();
	if (monitorProfile == 0)
		monitorProfile = cmsCreate_sRGBProfile ();

	if (ip)
		tmp_inputProfile = inputProfile;
	else
		tmp_inputProfile = cmsCreate_sRGBProfile ();

	if (op)
		tmp_outputProfile = outputProfile;
	else
		tmp_outputProfile = cmsCreate_sRGBProfile ();
	//wInput = (EXTRA_SH(0)|CHANNELS_SH(3)|BYTES_SH(1)|COLORSPACE_SH(PT_RGB)|PLANAR_SH(FALSE));
	wInput = ComputeOutputFormatDescriptor (0,
						GetProfileColorSpace
						(tmp_inputProfile));

	if (op && mp)
	{
		// soft proof transform    
		fprintf (stderr,
			 "soft-proofing output profile and monitor profile gamut= %d\n",
			 gamut);
		OutputColorSpace = GetProfileColorSpace (monitorProfile);
		wOutput =
			ComputeOutputFormatDescriptor (wInput,
						       OutputColorSpace);

		transform =
			cmsCreateProofingTransform (tmp_inputProfile, wInput,
						    monitorProfile, wOutput,
						    outputProfile, intent,
						    proofIntent,
						    (gamut
						     ? (cmsFLAGS_SOFTPROOFING
							| cmsFLAGS_GAMUTCHECK)
						     :
						     (cmsFLAGS_SOFTPROOFING)));
	}
	else if (mp)
	{
		// normal transform input to monitor
		fprintf (stderr,
			 "standard transform input profile to monitor profile\n");
		OutputColorSpace = GetProfileColorSpace (monitorProfile);
		wOutput =
			ComputeOutputFormatDescriptor (wInput,
						       OutputColorSpace);

		transform = cmsCreateTransform (tmp_inputProfile, wInput,
						monitorProfile, wOutput,
						intent, 0);
	}
	else
	{
		// normal transform input to printer
		fprintf (stderr,
			 "standard transform input profile to output profile\n");
		OutputColorSpace = GetProfileColorSpace (tmp_outputProfile);
		wOutput =
			ComputeOutputFormatDescriptor (wInput,
						       OutputColorSpace);

		transform = cmsCreateTransform (tmp_inputProfile, wInput,
						tmp_outputProfile, wOutput,
						intent, 0);
	}

	if (!ip)
		cmsCloseProfile (tmp_inputProfile);
	if (!op)
		cmsCloseProfile (tmp_outputProfile);

}

void
dumpICCtagASCII (cmsHPROFILE profile, icTagSignature tag)
{

	char text[2048] = "\0\0\0";
	int retval = 0;

	if ((retval = cmsReadICCText (profile, tag, text)) > 0)
	{
		fprintf (stderr, "%d chars= .%s.\n", retval, text);
	}
	else
	{
		//fprintf(stderr,"error reading tag\n");
	}
}

cmsHPROFILE
chooseProfile (char pid)
{
	switch (pid)
	{
	case 'i':
		return inputProfile;
		break;
	case 'o':
		return outputProfile;
		break;
	case 'm':
		return monitorProfile;
		break;
	default:
		fprintf (stderr, "chooseProfile:  bad profile ID %c\n", pid);
		return (cmsHPROFILE) NULL;
		break;
	}
}

char *
getICCtagText (char pid, icTagSignature tag)
{
	static char text[2048] = "\0\0\0";
	cmsHPROFILE hprofile;
	hprofile = chooseProfile (pid);
	if (cmsIsTag (hprofile, tag))
		cmsReadICCText (hprofile, tag, text);
	return text;
}

void 
setColorant(cmsHPROFILE hprofile, icTagSignature tag, double cols[3]) {
  cmsCIEXYZ ciexyz;

  ciexyz.X = cols[X_OFF];
  ciexyz.Y = cols[Y_OFF];
  ciexyz.Z = cols[Z_OFF];

  cmsAddTag(hprofile,tag,(LPcmsCIEXYZ) &ciexyz);
}

BOOL
getColorants (char pid, double Rcols[3], double Gcols[3], double Bcols[3])
{
	cmsHPROFILE hprofile;
	cmsCIEXYZTRIPLE ciexyzTriple;

	hprofile = chooseProfile (pid);
	if (cmsTakeColorants ((LPcmsCIEXYZTRIPLE) & ciexyzTriple, hprofile))
	{
		Rcols[X_OFF] = ciexyzTriple.Red.X;
		Rcols[Y_OFF] = ciexyzTriple.Red.Y;
		Rcols[Z_OFF] = ciexyzTriple.Red.Z;

		Gcols[X_OFF] = ciexyzTriple.Green.X;
		Gcols[Y_OFF] = ciexyzTriple.Green.Y;
		Gcols[Z_OFF] = ciexyzTriple.Green.Z;

		Bcols[X_OFF] = ciexyzTriple.Blue.X;
		Bcols[Y_OFF] = ciexyzTriple.Blue.Y;
		Bcols[Z_OFF] = ciexyzTriple.Blue.Z;
	}
	else
		return FALSE;

	return TRUE;
}

void 
setMediaPt(cmsHPROFILE hprofile, icTagSignature tag, double pt[3]) {
  cmsCIEXYZ mediaPt;

  mediaPt.X = pt[0];
  mediaPt.Y = pt[1];
  mediaPt.Z = pt[2];

  cmsAddTag(hprofile,tag,(LPcmsCIEXYZ) &mediaPt);
}

BOOL
getMediaPt (char ptId, char pid, double pt[3])
{
	cmsHPROFILE hprofile;
	cmsCIEXYZ rslt;
	BOOL retval;

	hprofile = chooseProfile (pid);

	switch (ptId)
	{
	case 'w':
		retval = cmsTakeMediaWhitePoint ((LPcmsCIEXYZ) & rslt,
						 hprofile);
		break;
	case 'b':
		retval = cmsTakeMediaBlackPoint ((LPcmsCIEXYZ) & rslt,
						 hprofile);
		break;
	default:
		fprintf (stderr, "getMPt: bad media point ID %c\n", ptId);
		return FALSE;
		break;
	}

	if (retval)
	{
		pt[X_OFF] = rslt.X;
		pt[Y_OFF] = rslt.Y;
		pt[Z_OFF] = rslt.Z;
	}

	return retval;

}

void
prtICCinfo (char pid)
{
	cmsHPROFILE pro;

	fprintf (stderr, "\n");

	pro = chooseProfile (pid);

	fprintf (stderr, "mfg desc - ");
	dumpICCtagASCII (pro, icSigDeviceMfgDescTag);

	fprintf (stderr, "model desc - ");
	dumpICCtagASCII (pro, icSigDeviceModelDescTag);

	fprintf (stderr, "copyright - ");
	dumpICCtagASCII (pro, icSigCopyrightTag);

	fprintf (stderr, "profile desc - ");
	dumpICCtagASCII (pro, icSigProfileDescriptionTag);


	fprintf (stderr, "LCMS product name: ");
	fprintf (stderr, "%s\n", cmsTakeProductName (pro));

	fprintf (stderr, "LCMS profile info: \n");
	fprintf (stderr, "%s\n", cmsTakeProductInfo (pro));
}

char *
getICC_ProdName (char pid)
{

	cmsHPROFILE pro;

	pro = chooseProfile (pid);

	return (char *) cmsTakeProductName (pro);

}

char *
getICC_ProfInfo (char pid)
{

	cmsHPROFILE pro;

	pro = chooseProfile (pid);

	return (char *) cmsTakeProductInfo (pro);
}


icProfileClassSignature
getDeviceClass (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsGetDeviceClass (pro);
}

icColorSpaceSignature
getColSpace (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsGetColorSpace (pro);
}

icColorSpaceSignature
getPCS (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsGetPCS (pro);
}

DWORD
getProfileICCversion (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsGetProfileICCversion (pro);
}

LPBYTE
getProfileICCid (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return (LPBYTE) cmsTakeProfileID (pro);
}

int
getIntent (char pid)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsTakeRenderingIntent (pro);
}

BOOL
getCharTargetData (char pid, char **data, size_t * len)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsTakeCharTargetData (pro, data, len);
}

LPLUT
getLUT (char pid, icTagSignature sig)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsReadICCLut (pro, sig);
}

LPGAMMATABLE
getGT (char pid, icTagSignature sig)
{
	cmsHPROFILE pro;
	pro = chooseProfile (pid);
	return cmsReadICCGamma (pro, sig);
}

void
lab2xyz(GdkColor *gcolor) 
{
	// coded directly from LabWarp mainicm.pas
	// 2 degree observer, D65 illuminant
	double refX=95.047;
	double refY=100.0;
	double refZ=108.883;
	double vX,vY,vZ,pvX,pvY,pvZ;

	vY = (((double) gcolor->red) + 16.0) / 116.0;
	vX = (((double) gcolor->green) / 500.0) + vY;  
	vZ = vY - (((double) gcolor->blue) / 200.0);
	
	if (vZ < 0) { vZ = 0.0001; }
	if (vY < 0) { vY = 0.0001; }
	if (vX < 0) { vX = 0.0001; }
	
	pvY = pow(vY,3);
	pvX = pow(vX,3);
	pvZ = pow(vZ,3);
	
	vY = ( pvY > 0.008856 ? pvY : (vY - (16.0 / 116.0)) / 7.787 );  
	vX = ( pvX > 0.008856 ? pvX : (vX - (16.0 / 116.0)) / 7.787 );
	vZ = ( pvX > 0.008856 ? pvZ : (vZ - (16.0 / 116.0)) / 7.787 );
	
	gcolor->red   = (WORD) (refX * vX) + 0.5;
	gcolor->green = (WORD) (refY * vY) + 0.5;
	gcolor->blue  = (WORD) (refZ * vZ) + 0.5;		
}

void 
xyz2rgb(GdkColor *gcolor,double gamma) 
{
	// coded directly from LabWarp mainicm.pas
	double vX,vY,vZ,vR,vG,vB;
	
	vX = ((double) gcolor->red) / 100.0;
	vY = ((double) gcolor->green) / 100.0;
	vZ = ((double) gcolor->blue) / 100.0;
	
	vR = (vX *  3.2410) + (vY * -1.5374) + (vZ * -0.4986);
	vG = (vX * -0.9692) + (vY *  1.8760) + (vZ *  0.0416);
	vB = (vX *  0.0556) + (vY * -0.2040) + (vZ *  1.0570);
	
	vR = ( vR >0.00304 ? (1.055 * pow(vR,(1.0/gamma))) - 0.055 : 12.92 * vR );
	vG = ( vG >0.00304 ? (1.055 * pow(vG,(1.0/gamma))) - 0.055 : 12.92 * vG );
	vB = ( vB >0.00304 ? (1.055 * pow(vB,(1.0/gamma))) - 0.055 : 12.92 * vB );

	if (vR < 0.0) { vR = 0.0; }
	if (vG < 0.0) { vG = 0.0; }
	if (vB < 0.0) { vB = 0.0; }
	
	if (vR > 1.0) { vR = 1.0; }
	if (vG > 1.0) { vG = 1.0; }
	if (vB > 1.0) { vB = 1.0; }

	gcolor->red   = (WORD) ( 0.5 + (vR * 0xffff));
	gcolor->green = (WORD) ( 0.5 + (vG * 0xffff));
	gcolor->blue  = (WORD) ( 0.5 + (vB * 0xffff));
}


void
lab2xyzA(double color[3]) 
{
	// coded directly from LabWarp mainicm.pas
	// 2 degree observer, D65 illuminant
	double refX=95.047;
	double refY=100.0;
	double refZ=108.883;
	double vX,vY,vZ,pvX,pvY,pvZ;

	vY = (color[0] + 16.0) / 116.0;   // Lr
	vX = (color[1] / 500.0) + vY;     // Ar
	vZ = vY - (color[2] / 200.0);     // Br
	
	if (vZ < 0) { vZ = 0.0001; }
	if (vY < 0) { vY = 0.0001; }
	if (vX < 0) { vX = 0.0001; }
	
	pvY = pow(vY,3);
	pvX = pow(vX,3);
	pvZ = pow(vZ,3);
	
	vY = ( pvY > 0.008856 ? pvY : (vY - (16.0 / 116.0)) / 7.787 );  
	vX = ( pvX > 0.008856 ? pvX : (vX - (16.0 / 116.0)) / 7.787 );
	vZ = ( pvZ > 0.008856 ? pvZ : (vZ - (16.0 / 116.0)) / 7.787 );
	
	/*
	color[0]   = (WORD) (refX * vX) + 0.5;
	color[1] = (WORD) (refY * vY) + 0.5;
	color[2]  = (WORD) (refZ * vZ) + 0.5;		
	*/
	color[0] = refX * vX;
	color[1] = refY * vY;
	color[2] = refZ * vZ;		
}

void 
xyz2rgbA(double color[3],double gamma) 
{
	// coded directly from LabWarp mainicm.pas
	double vX,vY,vZ,vR,vG,vB;
	
	vX = color[0] / 100.0;
	vY = color[1] / 100.0;
	vZ = color[2] / 100.0;
	
	vR = (vX *  3.2410) + (vY * -1.5374) + (vZ * -0.4986);
	vG = (vX * -0.9692) + (vY *  1.8760) + (vZ *  0.0416);
	vB = (vX *  0.0556) + (vY * -0.2040) + (vZ *  1.0570);
	
	vR = ( vR >0.00304 ? (1.055 * pow(vR,(1.0/gamma))) - 0.055 : 12.92 * vR );
	vG = ( vG >0.00304 ? (1.055 * pow(vG,(1.0/gamma))) - 0.055 : 12.92 * vG );
	vB = ( vB >0.00304 ? (1.055 * pow(vB,(1.0/gamma))) - 0.055 : 12.92 * vB );

	if (vR < 0.0) { vR = 0.0; }
	if (vG < 0.0) { vG = 0.0; }
	if (vB < 0.0) { vB = 0.0; }
	
	if (vR > 1.0) { vR = 1.0; }
	if (vG > 1.0) { vG = 1.0; }
	if (vB > 1.0) { vB = 1.0; }
	

	/*
	color[0]  = (WORD) ( 0.5 + (vR * 0xffff));
	color[1]  = (WORD) ( 0.5 + (vG * 0xffff));
	color[2]  = (WORD) ( 0.5 + (vB * 0xffff));
	*/

	color[0]  = vR * d(0xffff);
	color[1]  = vG * d(0xffff);
	color[2]  = vB * d(0xffff);
}


void 
rgb2xyzA(double color[3], double gamma) {
  // coded directly from LabWarp mainicm.pas
  // 2 deg observer, D65 Illuminant
  double vR,vG,vB;

  vR = color[0] / d(0xffff);
  vG = color[1] / d(0xffff);
  vB = color[2] / d(0xffff);

  
  vR = ( vR > 0.03928 ? pow( (vR + 0.055) / 1.055, gamma) : vR / 12.92 );
  vG = ( vG > 0.03928 ? pow( (vG + 0.055) / 1.055, gamma) : vG / 12.92 );
  vB = ( vB > 0.03928 ? pow( (vB + 0.055) / 1.055, gamma) : vB / 12.92 );
  
  /*
  vR = pow(vR,gamma);
  vG = pow(vG,gamma);
  vB = pow(vB,gamma);
  */

  vR *= 100.0;
  vG *= 100.0;
  vB *= 100.0;
  
  color[0] = (vR * 0.4124) + (vG * 0.3576) + (vB * 0.1805);
  color[1] = (vR * 0.2126) + (vG * 0.7152) + (vB * 0.0722);
  color[2] = (vR * 0.0193) + (vG * 0.1192) + (vB * 0.9505);

}


void
xyz2labA(double color[3]) {
  // coded directly from LabWarp mainicm.pas
  double rX,rY,rZ,vX,vY,vZ;

  rX =  95.047;       //Observer. = 2, Illuminant = D65
  rY = 100.000;
  rZ = 108.883;

  vX = color[0] / rX;        //inp_X = From 0 to ref_X
  vY = color[1] / rY;        //inp_Y = From 0 to ref_Y
  vZ = color[2] / rZ;        //inp_Z = From 0 to ref_Z

  vX = (vX > 0.008856 ? pow(vX,(1.0/3.0)) : (7.787 * vX) + (16.0 / 116.0) );
  vX = (vY > 0.008856 ? pow(vY,(1.0/3.0)) : (7.787 * vY) + (16.0 / 116.0) );
  vX = (vZ > 0.008856 ? pow(vZ,(1.0/3.0)) : (7.787 * vZ) + (16.0 / 116.0) );

  color[0] = ( 116.0 * vY ) - 16.0;
  color[1] = 500 * ( vX - vY );
  color[2] = 200 * ( vY - vZ );
  
}


#include "update_flags.h"

UL update_flags1,update_flags2;

void 
set_update_flags(char *wname) 
{
  switch (wname[0]) {
  case 'a':                   // a2b
    switch (wname[3]) {
    case '0':                 // a2b0
      switch (wname[5]) {
      case '1':               // a2b0L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B0L1C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B0L1C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B0L1C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B0L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b0L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // a2b0L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B0L2C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B0L2C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B0L2C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B0L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b0L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // a2b0clut
	update_flags1 |= ( A2B0CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: a2b0 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '1':                 //a2b1
      switch (wname[5]) {
      case '1':               // a2b1L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B1L1C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B1L1C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B1L1C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B1L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b1L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // a2b1L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B1L2C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B1L2C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B1L2C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B1L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b1L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // a2b1clut
	update_flags1 |= ( A2B1CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: a2b1 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '2':                // a2b2
      switch (wname[5]) {
      case '1':               // a2b2L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B2L1C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B2L1C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B2L1C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B2L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b2L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // a2b2L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( A2B2L2C1);
	  break;
	case '2':
	  update_flags1 |= ( A2B2L2C2);
	  break;
	case '3':
	  update_flags1 |= ( A2B2L2C3);
	  break;
	case '4':
	  update_flags1 |= ( A2B2L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: a2b2L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // a2b2clut
	update_flags1 |= ( A2B2CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: a2b2 widget %s not recognized\n",wname);
	break;
      }
      break;
    default:
      fprintf(stderr,"set_update_flags: a2b widget %s not recognized\n",wname);
      break;
    }
    break;
  case 'b':                   // b2a
    switch (wname[3]) {
    case '0':                 // b2a0
      switch (wname[5]) {
      case '1':               // b2a0L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A0L1C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A0L1C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A0L1C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A0L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a0L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // b2a0L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A0L2C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A0L2C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A0L2C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A0L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a0L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // b2a0clut
	update_flags1 |= ( B2A0CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: b2a0 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '1':                 //b2a1
      switch (wname[5]) {
      case '1':               // b2a1L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A1L1C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A1L1C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A1L1C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A1L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a1L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // b2a1L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A1L2C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A1L2C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A1L2C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A1L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a1L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // b2a1clut
	update_flags1 |= ( B2A1CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: b2a1 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '2':                // b2a2
      switch (wname[5]) {
      case '1':               // b2a2L1
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A2L1C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A2L1C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A2L1C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A2L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a2L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // b2a2L2
	switch (wname[7]) {
	case '1':
	  update_flags1 |= ( B2A2L2C1);
	  break;
	case '2':
	  update_flags1 |= ( B2A2L2C2);
	  break;
	case '3':
	  update_flags1 |= ( B2A2L2C3);
	  break;
	case '4':
	  update_flags1 |= ( B2A2L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: b2a2L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // b2a2clut
	update_flags1 |= ( B2A2CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: b2a2 widget %s not recognized\n",wname);
	break;
      }
      break;
    default:
      fprintf(stderr,"set_update_flags: b2a widget %s not recognized\n",wname);
      break;
    }

    break;
  case 'g':                   // gmut  (gamut)
    switch (wname[5]) {
      
    case '1':               // gmutL1
      switch (wname[7]) {
      case '1':
	update_flags1 |= ( GMUTL1C1);
	break;
      case '2':
	update_flags1 |= ( GMUTL1C2);
	break;
      case '3':
	update_flags1 |= ( GMUTL1C3);
	break;
      case '4':
	update_flags1 |= ( GMUTL1C4);
	break;
      default:
	fprintf(stderr,"set_update_flags: gmutL1 widget %s not recognized\n",wname);
	break;
      }
      break;
    case '2':               // gmutL2
      switch (wname[7]) {
      case '1':
	update_flags1 |= ( GMUTL2C1);
	break;
      case '2':
	update_flags1 |= ( GMUTL2C2);
	break;
      case '3':
	update_flags1 |= ( GMUTL2C3);
	break;
      case '4':
	update_flags1 |= ( GMUTL2C4);
	break;
      default:
	fprintf(stderr,"set_update_flags: gmutL2 widget %s not recognized\n",wname);
	break;
      }
      break;
    case 'l':               // gmutclut
      update_flags1 |= ( GMUTCLUT);
      break;
    default:
      fprintf(stderr,"set_update_flags: gmut widget %s not recognized\n",wname);
      break;
    }
    break;
  
    // preview
  case 'p':                   // prv
    switch (wname[3]) {
    case '0':                 // prv00
      switch (wname[5]) {
      case '1':               // prv0L1
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV0L1C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV0L1C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV0L1C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV0L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv0L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // prv0L2
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV0L2C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV0L2C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV0L2C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV0L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv0L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // prv0clut
	update_flags2 |= ( PRV0CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: prv0 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '1':                 //prv1
      switch (wname[5]) {
      case '1':               // prv1L1
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV1L1C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV1L1C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV1L1C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV1L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv1L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // prv1L2
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV1L2C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV1L2C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV1L2C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV1L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv1L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // prv1clut
	update_flags2 |= ( PRV1CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: prv1 widget %s not recognized\n",wname);
	break;
      }
      break;

    case '2':                // prv2
      switch (wname[5]) {
      case '1':               // prv2L1
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV2L1C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV2L1C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV2L1C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV2L1C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv2L1 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case '2':               // prv2L2
	switch (wname[7]) {
	case '1':
	  update_flags2 |= ( PRV2L2C1);
	  break;
	case '2':
	  update_flags2 |= ( PRV2L2C2);
	  break;
	case '3':
	  update_flags2 |= ( PRV2L2C3);
	  break;
	case '4':
	  update_flags2 |= ( PRV2L2C4);
	  break;
	default:
	  fprintf(stderr,"set_update_flags: prv2L2 widget %s not recognized\n",wname);
	  break;
	}
	break;
      case 'l':               // prv2clut
	update_flags2 |= ( PRV2CLUT);
	break;
      default:
	fprintf(stderr,"set_update_flags: prv2 widget %s not recognized\n",wname);
	break;
      }
      break;
    default:
      fprintf(stderr,"set_update_flags: prv widget %s not recognized\n",wname);
      break;
    }
    break;
    
  case 't':                  // trc
    switch (wname[5]) {
    case 'd':                // trcRed
      update_flags2 |= TRCRED  ;
      break;
    case 'n':                // trcGrn
      update_flags2 |= TRCGRN  ;
      break;
    case 'u':                // trcBlu
      update_flags2 |= TRCBLU  ;
      break;
    case 'y':                // trcGry
      update_flags2 |= TRCGRY  ;
      break;
    default:
      fprintf(stderr,"set_update_flags: trc widget %s not recognized\n",wname);
      break;
    }
    break;


  default:
    fprintf(stderr,"set_update_flags: widget %s not recognized\n",wname);
    break;
  }

  
  //fprintf(stderr,"suf %s %0llx %0llx\n",wname,update_flags1,update_flags2);
}
